@@ -564,15 +564,7 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_WIDTH),
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_HEIGHT),
     MAKE_ENTRY(NV_CTRL_GVIO_VIDEO_FORMAT_REFRESH_RATE),
-    MAKE_ENTRY(NV_CTRL_GPU_OVERCLOCKING_STATE),
-    MAKE_ENTRY(NV_CTRL_GPU_2D_CLOCK_FREQS),
-    MAKE_ENTRY(NV_CTRL_GPU_3D_CLOCK_FREQS),
-    MAKE_ENTRY(NV_CTRL_GPU_DEFAULT_2D_CLOCK_FREQS),
-    MAKE_ENTRY(NV_CTRL_GPU_DEFAULT_3D_CLOCK_FREQS),
     MAKE_ENTRY(NV_CTRL_GPU_CURRENT_CLOCK_FREQS),
-    MAKE_ENTRY(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS),
-    MAKE_ENTRY(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION),
-    MAKE_ENTRY(NV_CTRL_GPU_OPTIMAL_CLOCK_FREQS_DETECTION_STATE),
     MAKE_ENTRY(NV_CTRL_FLATPANEL_CHIP_LOCATION),
     MAKE_ENTRY(NV_CTRL_FLATPANEL_LINK),
     MAKE_ENTRY(NV_CTRL_FLATPANEL_SIGNAL),
@@ -612,7 +604,6 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_HWOVERLAY),
     MAKE_ENTRY(NV_CTRL_NUM_GPU_ERRORS_RECOVERED),
     MAKE_ENTRY(NV_CTRL_REFRESH_RATE_3),
-    MAKE_ENTRY(NV_CTRL_ONDEMAND_VBLANK_INTERRUPTS),
     MAKE_ENTRY(NV_CTRL_GPU_POWER_SOURCE),
     MAKE_ENTRY(NV_CTRL_GPU_CURRENT_PERFORMANCE_MODE),
     MAKE_ENTRY(NV_CTRL_GLYPH_CACHE),
@@ -692,6 +683,7 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_GVI_TEST_MODE),
     MAKE_ENTRY(NV_CTRL_COLOR_SPACE),
     MAKE_ENTRY(NV_CTRL_COLOR_RANGE),
+    MAKE_ENTRY(NV_CTRL_SHOW_GSYNC_VISUAL_INDICATOR),
     MAKE_ENTRY(NV_CTRL_GPU_SCALING_DEFAULT_TARGET),
     MAKE_ENTRY(NV_CTRL_GPU_SCALING_DEFAULT_METHOD),
     MAKE_ENTRY(NV_CTRL_DITHERING_MODE),
@@ -742,5 +734,8 @@ static AttrEntry attr_table[] = {
     MAKE_ENTRY(NV_CTRL_MULTIGPU_MASTER_POSSIBLE),
     MAKE_ENTRY(NV_CTRL_GPU_POWER_MIZER_DEFAULT_MODE),
     MAKE_ENTRY(NV_CTRL_XV_SYNC_TO_DISPLAY_ID),
+    MAKE_ENTRY(NV_CTRL_CURRENT_XV_SYNC_TO_DISPLAY_ID),
+    MAKE_ENTRY(NV_CTRL_PALETTE_UPDATE_EVENT),
+    MAKE_ENTRY(NV_CTRL_GSYNC_ALLOWED),
     { -1, NULL, NULL }
 };
